# -*- coding: utf-8 -*-
# copyright: (c) 2021 by Jesse Johnson.
# license: MPL-2.0, see LICENSE for more details.
"""Provide CLI methods for Proman source tools."""

from proman_versioning import get_release_controller

controller = get_release_controller()
